/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <errno.h>
#include <assert.h>
#include <termios.h>


#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/wait.h>
#include <signal.h>
#include <netdb.h>

#include "logfile_io.h"


/* -------------------------------------------------------------------- */

#define INIT_SEND   0
#define CLIENT_SEND 1
#define SERVER_SEND 2


static int flog = -1;
static int sender_state = INIT_SEND;
static int new_paragraph = 0;


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
void init_logfile(char *logfile)
{
	flog = open(logfile, O_WRONLY|O_TRUNC|O_CREAT, S_IRWXU|S_IRGRP|S_IROTH);
	if (flog == -1)
	{
		fprintf(stderr, "Error: opening file '%s'\n", logfile);
		exit(-1);
	}

	new_paragraph = 0;
}

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
void log_client_send(void)
{
	if (flog != -1)
	{
		if (sender_state != CLIENT_SEND)
		{	
			/* Just switched */

			if ((sender_state != INIT_SEND) &&
			    (!new_paragraph))
			{
				write(flog, "\"\n", strlen("\"\n"));
			}

			write(flog, "Client --> \"", strlen("Client --> \""));
			new_paragraph = 0;
		}

		sender_state = CLIENT_SEND;
	}
}

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
void log_server_send(void)
{
	if (flog != -1)
	{
		if (sender_state != SERVER_SEND)
		{	
			/* Just switched */

			if ((sender_state != INIT_SEND) &&
			    (!new_paragraph))
			{
				write(flog, "\"\n", strlen("\"\n"));
			}

			write(flog, "Server --> \"", strlen("Server --> \""));
			new_paragraph = 0;
		}

		sender_state = SERVER_SEND;
	}
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
void log_character(char c)
{
	char buf[64];


	if (flog != -1)
	{
		if (new_paragraph)
		{
			write(flog, "           \"", strlen("           \""));
			new_paragraph = 0;
		}

		if (c == '\n') 
		{

			write(flog, "\\n\"\n", strlen("\\n\"\n"));	/* Ouput data to logfile */
			new_paragraph = 1;
		}
		else
		if (c == '\r')
		{
			write(flog, "\\r", strlen("\\r"));	/* Ouput data to logfile */
		}
		else
		if ((int)c < 32)
		{
			sprintf(buf, "\\0%02X", (int)c);

			write(flog, buf, strlen(buf));		/* Ouput data to logfile */
		}
		else
		{
			write(flog, &c, 1);	/* Ouput data to logfile */
		}
	}
}

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
void close_logfile(void)
{
	if (flog != -1)
	{
		close(flog);
	}
}
